﻿using System;
using System.Collections.ObjectModel;
using System.Drawing.Imaging;
using System.Text;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using OpenQA.Selenium;
using OpenQA.Selenium.Firefox;

namespace SeleniumTest
{
    [TestFixture]
    public class SeleniumExample
    {
        private FirefoxDriver _driver;
        private FirefoxDriver _noJavaScriptDriver;

        #region Setup

        [SetUp]
        public void Setup()
        {
            _driver = new FirefoxDriver();
            _driver.Manage().Timeouts().ImplicitlyWait(new TimeSpan(0, 0, 30));
        }

        /// <summary>
        /// Creates an instance of the FirefoxProfile with Javascript disabled
        /// </summary>
        public void StartJavascriptDisabledDriver()
        {
            FirefoxProfile profile = new FirefoxProfile();
            profile.SetPreference("javascript.enabled", false);
            _noJavaScriptDriver = new FirefoxDriver(profile);
            _noJavaScriptDriver.Manage().Timeouts().ImplicitlyWait(new TimeSpan(0, 0, 30));
        }

        #endregion

        #region Tests

        [Test]
        public void SearchResults_ShouldHaveCorrectPageTitle()
        {
            // Navigate
            _driver.Navigate().GoToUrl("http://www.deanhume.com");

            IWebElement searchBox = _driver.FindElement(By.Name("searchValue"));
            searchBox.SendKeys("moq");
            searchBox.Submit();

            Assert.AreEqual("Dean Hume - Search Results - moq", _driver.Title);
        }

        [Test]
        public void SearchResults_ShouldHaveGoogleAnalyticsScripts()
        {
            // Navigate
            _driver.Navigate().GoToUrl("http://www.deanhume.com");

            bool containsGoogleAnalytics = false;
            bool containsGoogleTracking = false;

            ReadOnlyCollection<IWebElement> scriptElements = _driver.FindElements(By.TagName("script"));

            // Loop through the results and confirm that we have both Google scripts
            foreach (var element in scriptElements)
            {
                if (element.GetAttribute("src") == "http://www.deanhume.com/Zip/ZipThis?Path=/Content/js/googleTracking.min.js")
                {
                    containsGoogleTracking = true;
                }

                if (element.GetAttribute("src") == "http://www.deanhume.com/Zip/ZipThis?Path=/Content/js/googleAnalytics.min.js")
                {
                    containsGoogleAnalytics = true;
                }
            }


            Assert.That(containsGoogleTracking, Is.True);
            Assert.That(containsGoogleAnalytics, Is.True);
        }

        [Test]
        public void SearchResults_WithJavascriptDisabled_ShouldShowWarning()
        {
            StartJavascriptDisabledDriver();

            bool containsMessage = false;

            // Disable Javascript
            _noJavaScriptDriver.Navigate().GoToUrl("http://www.deanhume.com");

            // Check that the warning box appears
            ReadOnlyCollection<IWebElement> scriptElements = _noJavaScriptDriver.FindElements(By.TagName("table"));

            foreach (IWebElement element in scriptElements)
            {
                // Check if the box exists with the correct message.
                if (element.Text == "This site is best viewed with JavaScript enabled.")
                {
                    containsMessage = true;
                }

            }

            Assert.That(containsMessage, Is.True);

        }

        [Test]
        public void PageError_SavesScreenshot()
        {
            // Navigate our url
            const string urlToNavigateTo = "http://www.deanhume.com/ForceError";
            _driver.Navigate().GoToUrl(urlToNavigateTo);
          
            // Check that the url's are correct and match
            if (urlToNavigateTo != _driver.Url)
            {
                SaveScreenShot("Error");
                Assert.Fail("Url's do not match");
            }
        }

        /// <summary>
        /// Saves a screenshot of the current error page
        /// </summary>
        public void SaveScreenShot(string fileName)
        {
            // Get the screenshot
            Screenshot screenshot = _driver.GetScreenshot();

            // Build up our filename
            StringBuilder filename = new StringBuilder(fileName);
            filename.Append("-");
            filename.Append(DateTime.Now.ToString("yyyy-MM-dd HH_mm_ss"));
            filename.Append(".png");

            // Save the image
            screenshot.SaveAsFile(filename.ToString(), ImageFormat.Png);
        }

        #endregion

        #region TearDown

        [TearDown]
        public void FixtureTearDown()
        {
            if (_driver != null) _driver.Close();
            if (_noJavaScriptDriver != null) _noJavaScriptDriver.Close();
        }

        #endregion
    }
}
